@@ -19,15 +19,16 @@ import utils
 
 
 __version__=json.loads(pkg_resources.resource_string('synapseclient', 'synapsePythonClient'))['latestVersion']
+CACHE_DIR=os.path.join(os.path.expanduser('~'), '.synapseCache', 'python')  #TODO change to /data when storing files as md5
+CONFIG_FILE=os.path.join(os.path.expanduser('~'), '.synapseConfig')
 
-CACHE_DIR='~/.synapseCache/python'  #TODO: this needs to be handled in a transparent way!
 
 class Synapse:
     """
     Python implementation for Synapse repository service client
     """    
-    def __init__(self, repoEndpoint='https://repo-prod.sagebase.org/repo/v1', 
-                 authEndpoint='https://auth-prod.sagebase.org/auth/v1', 
+    def __init__(self, repoEndpoint='https://repo-prod.prod.sagebase.org/repo/v1', 
+                 authEndpoint='https://auth-prod.prod.sagebase.org/auth/v1', 
                  serviceTimeoutSeconds=30, debug=False):
         '''Constructor of Synapse client
         params:
@@ -38,7 +39,21 @@ class Synapse:
         '''
 
         self.cacheDir = os.path.expanduser(CACHE_DIR)
-        self.headers = {'content-type': 'application/json', 'Accept': 'application/json'}
+        #create cacheDir if it does not exist
+        try:
+            os.makedirs(self.cacheDir)
+        except OSError as exception:
+            if exception.errno != os.errno.EEXIST:
+                raise
+
+        resp=requests.get(repoEndpoint, allow_redirects=False)
+        if resp.status_code==301:
+            repoEndpoint=resp.headers['location']
+        resp=requests.get(authEndpoint, allow_redirects=False)
+        if resp.status_code==301:
+            authEndpoint=resp.headers['location']
+
+        self.headers = {'content-type': 'application/json', 'Accept': 'application/json', 'request_profile':'False'}
 
         self.serviceTimeoutSeconds = serviceTimeoutSeconds 
         self.debug = debug
@@ -56,9 +71,6 @@ class Synapse:
         self.authEndpoint["location"] = parseResult.netloc
         self.authEndpoint["prefix"] = parseResult.path
         self.authEndpoint["protocol"] = parseResult.scheme
-        
-        self.request_profile = False
-        self.profile_data = None
 
 
     def _connect(self, endpoint):
@@ -81,36 +93,56 @@ class Synapse:
         return conn
 
 
+    def _storeTimingProfile(self, resp):
+        """Stores timint information for the last call if request_profile was set."""
+        if self.headers.get('request_profile', 'False')=='True':
+            profile_data = None
+            for k,v in resp.getheaders():
+                if k == "profile_response_object":
+                    profile_data = v
+                    break
+            self.profile_data = json.loads(base64.b64decode(profile_data))
+
+
     def printEntity(self, entity):
         """Pretty prints an entity."""
         print json.dumps(entity, sort_keys=True, indent=2)
 
 
-    def login(self, email, password):
+    def login(self, email=None, password=None):
         """
-        Authenticate and get session token
+        Authenticate and get session token by using (in order of preference):
+        1) supplied email and password
+        2) check for configuraton file
+        3) Use already existing session token
         """
         ## check version before logging in
         version_check()
 
         session_file = os.path.join(self.cacheDir, ".session")
-        if (None==email and None==password and os.path.exists(session_file)):
-            with open(session_file) as f:
-                sessionToken = f.read().strip()
-            self.sessionToken = sessionToken
-            self.headers["sessionToken"] = sessionToken
-            return sessionToken
 
-        if (None == email or None == password):
-            raise Exception("missing Synapse username or password")
 
-        # Disable profiling during login
-        # TODO: Check what happens if enabled
-        req_profile = None
-        if self.request_profile != None:
-            req_profile = self.request_profile
-            self.request_profile = False
-        
+        if (email==None or password==None): #Try to use config then session token
+            try:
+                import ConfigParser
+                config = ConfigParser.ConfigParser()
+                config.read(CONFIG_FILE)
+                email=config.get('authentication', 'username')
+                password=config.get('authentication', 'password')
+            except ConfigParser.NoSectionError:  #Authentication not defined in config reverting to session token
+                if os.path.exists(session_file):
+                    with open(session_file) as f:
+                        sessionToken = f.read().strip()
+                    self.sessionToken = sessionToken
+                    self.headers["sessionToken"] = sessionToken
+                    return sessionToken
+                else:
+                    raise Exception("LOGIN FAILED: no username/password, configuration or cached session token available")
+
+
+        # Disable profiling during login and proceed with authentication
+        self.headers['request_profile'], orig_request_profile='False', self.headers['request_profile']
+
         uri = self.authEndpoint["prefix"] + "/session"
         req = {"email":email, "password":password}
         
@@ -122,9 +154,8 @@ class Synapse:
         with open(session_file, "w") as f:
             f.write(self.sessionToken)
         os.chmod(session_file, stat.S_IRUSR | stat.S_IWUSR)
+        self.headers['request_profile'] = orig_request_profile
 
-        if req_profile != None:
-            self.request_profile = req_profile
 
 
     def getEntity(self, entity, endpoint=None):
@@ -151,20 +182,11 @@ class Synapse:
         if(self.debug):  print 'About to get %s' % (uri)
 
         entity = None
-        self.profile_data = None
         try:
             headers = self.headers
-            if self.request_profile:
-                headers["profile_request"] = "True"
             conn.request('GET', uri, None, headers)
             resp = conn.getresponse()
-            if self.request_profile:
-                profile_data = None
-                for k,v in resp.getheaders():
-                    if k == "profile_response_object":
-                        profile_data = v
-                        break
-                self.profile_data = json.loads(base64.b64decode(profile_data))
+            self._storeTimingProfile(resp)
             output = resp.read()
             if resp.status == 200:
                 if self.debug:
@@ -197,9 +219,9 @@ class Synapse:
 
         filename = os.path.join(self.cacheDir,entity['id'] ,pathComponents[-1])
         if os.path.exists(filename):
-            print filename, "cached"
+            #print filename, "cached"
             md5 = utils.computeMd5ForFile(filename)
-            if md5.hexdigest() != entity['md5']:
+            if md5.hexdigest() != entity.get('md5', ''):
                 print filename, "changed, redownloading"
                 utils.downloadFile(url, filename)
         else:
@@ -207,7 +229,6 @@ class Synapse:
             utils.downloadFile(url, filename)
 
         if entity['contentType']=='application/zip':
-            print 'unpacking...'
             ## Unpack file
             filepath=os.path.join(os.path.dirname(filename), os.path.basename(filename)+'_unpacked')
             #TODO!!!FIX THIS TO BE PATH SAFE!  DON'T ALLOW ARBITRARY UNZIPING
@@ -245,21 +266,12 @@ r        - `entity`: Either a string or dict representing and entity
         if(self.debug): print 'About to create %s with %s' % (uri, json.dumps(entity))
 
         storedEntity = None
-        self.profile_data = None
         
         try:
             headers = self.headers
-            if self.request_profile:
-                headers["profile_request"] = "True"
             conn.request('POST', uri, json.dumps(entity), headers)
             resp = conn.getresponse()
-            if self.request_profile:
-                profile_data = None
-                for k,v in resp.getheaders():
-                    if k == "profile_response_object":
-                        profile_data = v
-                        break
-                self.profile_data = json.loads(base64.b64decode(profile_data))              
+            self._storeTimingProfile(resp)
             output = resp.read()
             if resp.status == 201:
                 if self.debug:
@@ -306,21 +318,12 @@ r        - `entity`: Either a string or dict representing and entity
         if(self.debug): print 'About to query %s' % (queryStr)
 
         results = None
-        self.profile_data = None
 
         try:
             headers = self.headers
-            if self.request_profile:
-                headers["profile_request"] = "True"
             conn.request('GET', uri, None, headers)
             resp = conn.getresponse()
-            if self.request_profile:
-                profile_data = None
-                for k,v in resp.getheaders():
-                    if k == "profile_response_object":
-                        profile_data = v
-                        break
-                self.profile_data = json.loads(base64.b64decode(profile_data))
+            self._storeTimingProfile(resp)
             output = resp.read()
             if resp.status == 200:
                 if self.debug:
@@ -351,21 +354,12 @@ r        - `entity`: Either a string or dict representing and entity
 
         if (self.debug): print 'About to delete %s' % (uri)
 
-        self.profile_data = None
 
         try:
             headers = self.headers
-            if self.request_profile:
-                headers["profile_request"] = "True"
             conn.request('DELETE', uri, None, headers)
             resp = conn.getresponse()
-            if self.request_profile:
-                profile_data = None
-                for k,v in resp.getheaders():
-                    if k == "profile_response_object":
-                        profile_data = v
-                        break
-                self.profile_data = json.loads(base64.b64decode(profile_data))
+            self._storeTimingProfile(resp)
             output = resp.read()
             if resp.status != 204:
                 raise Exception('DELETE %s failed: %d %s %s' % (uri, resp.status, resp.reason, output))
@@ -397,18 +391,11 @@ r        - `entity`: Either a string or dict representing and entity
             putHeaders['ETag'] = entity['etag']
 
         storedEntity = None
-        self.profile_data = None
 
         try:
             conn.request('PUT', uri, json.dumps(entity), putHeaders)
             resp = conn.getresponse()
-            if self.request_profile:
-                profile_data = None
-                for k,v in resp.getheaders():
-                    if k == "profile_response_object":
-                        profile_data = v
-                        break
-                self.profile_data = json.loads(base64.b64decode(profile_data))
+            self._storeTimingProfile(resp)
             output = resp.read()
             # Handle both 200 and 204 as auth returns 204 for success
             if resp.status == 200 or resp.status == 204:
